%
%   This function validates the data and moves one interpolation
%    point to zero if necessary
%
%   Input:  interdata - interpolation data set
%
%   Output: interdata - modified interpolation data set
%           tozerotrans - transformation of the data set that
%                          brought one interpolation point to
%                          origin. If no transformation is needed
%                          the transformation u = z is returned.
%
function [interdata, tozerotrans, symtrans] = DataValid(interdata)

%
%   Check that z is in D and w in rhp
%
if max(abs(interdata.z)) > 1
    disp('In DataValid: all interpolation points should')
    disp('   belong to the open unit disc')
    return
end

%
%   Check that the number of spectral zeros is at most the number
%    interpolation points taken with multiplicity minus one.
%
if length(interdata.dpoly) > interdata.mkn+1
    disp('In DataValid: The length of the dissipative polynomial should be')
    disp('   less or equal to the number of interpolation points')
    disp('   taken with multiplicity minus one.')
    return
end

%
%   Move one interpolation point to origin if necessary. If one
%    interpolation point already is origin it will, be construction
%    of interdata be the first one. Right now we choose to move the
%    real point with the highest interpolation multiplicity.
%
if interdata.z(1) == 0
    % No transformation needed
    tozerotrans = GetBlTrans(1,0,0,1);
else
    % This z0 will correspond to real w since the data has been
    %  extended to self-conjugatedness.
    z0 = interdata.z(find(interdata.mw == max(interdata.mw(find(imag(interdata.z) == 0)))));
    
    % To be modified!!
    z0 = interdata.z(1);
    
    if isempty(z0)
        % No real interpolation point
        disp('In datavalid: at least one interpolation point has')
        disp('   to be real')
        return
    end
    tozerotrans = GetBlTrans(1,-z0,-z0,1);
    interdata = BilinearDomainData(tozerotrans,interdata);
end

%
%   Make the interpolation data self-conjugated
%
interdata = SelfConj(interdata); 
% if interdata.W{1}{1} == interdata.W{1}{1}'
%     symtrans.U = eye(interdata.m);
%     symtrans.V = symtrans.U;
% else
%     [symtrans.U,S,symtrans.V] = svd(interdata.W{1}{1});
%     for k = 1:interdata.ntilde
%         for l = 1:length(interdata.W{k})
%             W{k}{l} = symtrans.U'*interdata.W{k}{l}*symtrans.V;
%         end
%     end
%     interdata = GetInterData(interdata.z,W,interdata.speczeros,interdata.maxdistancesp);   
% end