%
%   This script provide an interface to solve 
%   the Nevanlinna-Pick interpolation problem with degree constraint, 
%   including boundary interpolation cases.
%
format compact
warning off
addpath DrawFiles/ InputFiles/ TransformFiles/ HomotopyFiles/

%
%   Main loop
%
mainflag = 0;
while not(mainflag)
    try DomainRadius; catch DomainRadius = 1;                                  end
    try DomainRegion; catch DomainRegion = 'D';                                end  
    try Domain;       catch Domain = [num2str(DomainRadius) '*' DomainRegion]; end
    try RangeRadius;  catch RangeRadius = 1;                                   end
    try RangeRegion;  catch RangeRegion = 'D';                                 end
    try Range;        catch Range = [num2str(RangeRadius) '*' RangeRegion];    end    
    try z;            catch z = [];                                            end
    try w;            catch w = [];                                            end     
    
    mainoptions = { 'Input Data Menu',...
            'Computation of a solution',...
            'Load Data',...
            'Save Data',...
            'Display Results',...
            'Clear Variables',...
            'Exit' };
    chosenmainoption = DrawMenu('Main Menu',mainoptions);
    switch chosenmainoption
        %
        %     Load and modify data
        %
   case 'Input Data Menu'
       inputoptions = { 'Domain and Range',...
               'Interpolation Data',...
               'Return to Main Menu' };
       %
       %   Input loop
       %
       inputflag = 0;
       while not(inputflag)
           choseninputoption = DrawMenu('Input Data Menu',inputoptions);
           switch choseninputoption
          case 'Domain and Range' 
              DRflag = 1;
              while DRflag
                  DRvaluevec = { Domain Range };
                  flag = DrawCurrentDR(DRvaluevec);
                  if flag
                      disp(' ')
                      disp('Input (1)-(3) !')
                      [DomainRegion,DomainRadius] = ...
                          DRInput(' Domain should be: ',DomainRegion,DomainRadius);
                      [RangeRegion,RangeRadius] = ... 
                          DRInput(' Range should be: ',RangeRegion,RangeRadius);
                      if DomainRegion ~= 'C+'
                          Domain = [num2str(DomainRadius) '*' DomainRegion];
                      else
                          Domain = DomainRegion;
                      end
                      if RangeRegion ~= 'C+'
                          Range = [num2str(RangeRadius) '*' RangeRegion];
                      else
                          Range = RangeRegion;
                      end
                  else
                      DRflag = 0;
                  end
              end
              
          case 'Interpolation Data'
	          IntDataflag = 1;
              while IntDataflag == 1
                  flag = DrawIntData(z,w,1,Domain,Range);
                  if flag
                  %
                  % Modify or add interpolation data
                  %
                  optionsA = { 'Modify the current data' ...
                        'Add data' ...
                        'Remove data' ...
                        'Return to Input Data Menu' };
                  chosenoptionA = DrawMenu('Change-Data Menu',optionsA,0);
                  switch chosenoptionA
                 case 'Modify the current data'
                     idxz = NumericalInput('Which z do you want to modify (none = 0)?',0);
                     if idxz > 0 & idxz <= length(z) & idxz-floor(idxz)==0
                         %
                         %  Valid input
                         %
                         z(idxz) = NumericalInput('Give z:',z(idxz));
                         idxw = NumericalInput('Which w do you want to modify (none = 0)?',0);
                         if idxw > 0 & idxw <= length(w{idxz}) & idxw-floor(idxw)==0
                             w{idxz}{idxw}= NumericalInput('Give w:',w{idxz}{idxw});
                         end
                     end
                             
                  case 'Add data'
                      zadd = NumericalInput('Give an interpolation point:',[]);
                      if not(isempty(zadd))
                          z(end+1) = zadd;
                          waddflag = 1; k = 1;
                          while waddflag
                              wadd = NumericalInput('Give an interpolation value:',[]);
                              if isempty(wadd)
                                  waddflag = 0;
                              else
                                  w{length(z)}{k} = wadd;
                                  k = k+1;
                              end
                          end
                      end
                      
                  case 'Remove data'
                      idxz = NumericalInput('Which z do you want to remove?',[]);
                      if not(isempty(idxz))
                          flag = input('Do you want to remove all corresponding w? (n)','s');
                          if strcmp(flag,'y')
                              z(idxz) = []; w(idxz) = [];
                          else
                              idxw = NumericalInput('Which w do you want to remove?',[]);
                              if not(isempty(idxw)) & idxw <= length(w{idxz})
                                  w{idxz}(idxw) = [];
                                  if isempty(w{idxz})
                                      z(idxz) = []; w(idxz) = [];
                                  end
                              end
                          end 
                      end
                    
                  case 'Return to Input Data Menu'
                      IntDataflag = 0;
                   end
              else 
                 IntDataflag = 0;
              end
          end   
         
     case 'Return to Main Menu'
         inputflag = 1;
      end
  end
    
   %
   %    Computation of a Nevanlinna-Pick interpolant
   %      
   case 'Computation of a solution'
       if isempty(z)
           disp('Input interpolation data!')
           disp('Press any key'), pause
       else
           
       % check the compatibility of domain and z-data
       zOKflag = 1;
       if (DomainRegion == 'C+' & min(real(z)) < 0) | ... 
               (DomainRegion == 'D' & length(Domain)& max(abs(z)) > DomainRadius) | ...
               (DomainRegion == 'Dc' & min(abs(z)) < DomainRadius)               
               zOKflag = 0;
       end
       if zOKflag == 0
           disp(['z must be in ' Domain ' !']), z
           disp('Modify z and/or Domain !')
           disp('Press any key'), pause
       else
            % bilinear transform (to the setting 1*D --> C+)
            if DomainRegion == 'C+'
                domaintrans = GetBlTrans(-1,1,1,1);
            elseif DomainRegion == 'D'
                domaintrans = GetBlTrans(1,0,0,DomainRadius);
            elseif DomainRegion == 'Dc'
                domaintrans = GetBlTrans(0,1,DomainRadius,0);
            end
            if RangeRegion == 'D'
                rangetrans = GetBlTrans(-1,RangeRadius,1,RangeRadius);
            elseif RangeRegion == 'Dc'
                rangetrans = GetBlTrans(RangeRadius,-1,RangeRadius,1);
            end
            
          lw = 0;
          for k = 1:length(z)
              lw = lw + length(w{k});
              if abs(imag(z(k)))>1e-10
                  lw = lw + length(w{k});
              end
          end             
          % input spectral factor
          specfact = NumericalInput(['Give spectral factor (polynomial coefficients!)'],[]);
          if isempty(specfact)
              dpoly = [];
          elseif length(specfact) > lw
              while length(specfact) > lw
                  disp('length(specfact) should be smaller!')
                  specfact = ...
                      NumericalInput(['Give spectral factor (polynomial coefficients!)'],[]);
              end
          else
              dpoly = GetDissPoly(specfact);                
              dpoly = [dpoly ; zeros(lw-length(dpoly),1)];
          end
          
            % data structure and transform
            interdata = GetInterData(z,w,dpoly,1); 
            interdata2 = BilinearDomainData(domaintrans,interdata);          
            if RangeRegion ~= 'C+'
                interdata3 = BilinearRangeData(rangetrans,interdata2);
            else
                interdata3 = interdata2;
            end
            
            % solvability check 
            [P,shrinkflag,shrinkfactor] = GetPickMatrix(SelfConj(interdata3)); 
            if min(real(eig(P))) <= 0
                disp('Pick matrix is not positive definite !')
                disp('There is no solution for this data !')
                real(eig(P))
                disp('Press any key'), pause
            else
                % computation of a solution and transform back
                interpolant3 = homotopy(interdata3);
                if RangeRegion ~= 'C+'
                    interpolant2 = BilinearRange(rangetrans,interpolant3);
                else
                    interpolant2 = interpolant3;
                end
                interpolant = BilinearDomain(domaintrans,interpolant2);
                
                % Display the solution
                er = GetIntError(interpolant,interdata);
                [num,den] = tfdata(interpolant.sysf,'v');
                sys = minreal(tf(fliplr(num),fliplr(den),1));
                if DomainRegion == 'C+'
                    sys.Ts = 0;
                end
                sys                
                disp(' ')
                disp(['Error: ' num2str(er)])
                pause        
            end
       end
       
       end
   %
   %    Load a previous design from a mat-file
   %
   case 'Load Data'
    files = dir;
    dirname = {};
    for k = 1:length(files)
      if files(k).isdir
          dispflag = 1;
          if length(files(k).name)==9
              if sum(files(k).name=='DrawFiles') == 9, dispflag = 0; end
          elseif length(files(k).name)==13
              if sum(files(k).name=='HomotopyFiles') == 13, dispflag = 0; end
          elseif length(files(k).name)==10
              if sum(files(k).name=='InputFiles') == 10, dispflag = 0; end
          elseif length(files(k).name)==14
              if sum(files(k).name=='TransformFiles') == 14, dispflag = 0; end
          end
          if dispflag
              dirname{end+1} = files(k).name;
          end
      end
    end
    dirname = DrawMenu('Available directories',dirname);
    files = dir([dirname '/*.mat']);
    availfiles = {};
    for k = 1:length(files)
      availfiles{k} = files(k).name;
    end
    availfiles{end+1} = 'Return to Main Menu';
    chosenfile = DrawMenu('Available files in directory',availfiles);
    if not(strcmp(chosenfile,'Return to Main Menu'))
     load([dirname '/' chosenfile])
    end
    mainflag = 0;
   
   %
   %    Save the current variables in a mat-file
   %
   case 'Save Data'
    files = dir;
    dirname = {};
    for k = 1:length(files)
      if files(k).isdir
          dispflag = 1;
          if length(files(k).name)==9
              if sum(files(k).name=='DrawFiles') == 9, dispflag = 0; end
          elseif length(files(k).name)==13
              if sum(files(k).name=='HomotopyFiles') == 13, dispflag = 0; end
          elseif length(files(k).name)==10
              if sum(files(k).name=='InputFiles') == 10, dispflag = 0; end
          elseif length(files(k).name)==14
              if sum(files(k).name=='TransformFiles') == 14, dispflag = 0; end
          end
          if dispflag
              dirname{end+1} = files(k).name;
          end
      end
    end
    dirname{end+1} = 'Make new directory';
    dirname = DrawMenu('Available directories',dirname);
    if strcmp(dirname,'Make new directory')
      dirname = StringInput('Give directory: ','DataMatFiles');
    end
    if not(isdir(dirname))
      mkdir(dirname);
    end
    filename = input('Give filename (empty doesn''t save): ','s');
    if not(isempty(filename))
      save([dirname '/' filename '.mat'])
    end          
          
   case 'Display Results'
       if isempty(specfact)
           interdatasub = SelfConj(interdata);
           specfact = poly(interdatasub.z);
           dpoly = GetDissPoly(specfact);
       end
       DrawIntData(z,w,0,Domain,Range)
       disp('***** Parameters *****')
       disp(['Spectral factor polynomial: [' num2str(specfact) ']'])
       disp(['Spectral zeros: ' num2str(roots(specfact)')])
       disp(['Dissipative polynomial: ' ... % d0+d1(z+z^(-1))+...+dn(z^n+z^(-n)): ')
       'd = [ ' num2str(dpoly') ' ]'])
       disp(' ')
       disp('***** Interpolant nsys/dsys *****')
       [nsys,dsys] = tfdata(sys,'v')
       disp(' ')
       disp(['Error: ' num2str(er)])
       pause
   case 'Clear Variables'
    %
    %   Clear loop
    %
    clearflag = 0;
    while not(clearflag)
      clc
      clearoptions = { 'Clear All Variables in the Workspace',...
		       'Clear Some Variables in the Workspace',...
		       'Return to Main Menu' };
      chosenclearoption = DrawMenu('Clear Menu',clearoptions,0);
      
      switch chosenclearoption
       
       case 'Clear All Variables in the Workspace'
        flag = input('Do you really want to clear all the variables? (y)','s');
        if strcmp(flag,'y') | isempty(flag)
	  clear all
            clearflag = 1;
            mainflag = 0;
        end             
       case 'Clear Some Variables in the Workspace'
	disp('Currently')
	who
	name = input('Which do you want to clear? ','s');
	try
	  eval(name);
	  clear(name);
	catch
	  disp([ name ' is not available in the workspace']), pause
	end
       case 'Return to Main Menu'
	clearflag = 1;
      end
    end    
          
   case 'Exit'
       %
       %     Leave the program
       %       
       mainflag = 1;
    end
end

%
%   Save the workspace in all cases
%
save DataMatFiles/DataLatest.mat