%
%   This function transforms back an interpolant g(z) to f(z) for
%    the bilinear transformation
%
%       G(z) = (aF(z) + bI)(cF(z) + dI)^-1
%
%       F(z) = (cG(z) - aI)^-1 (bI - dG(z))
%
%                      (ad - bc != 0)
%
%       Input: rangetrans - transformation parameters
%              interpolant - a system on state space form
%
%       Output: interpolantnew  - the transfoprmed system on state space form
%
function interpolant = BilinearRange(rangetrans,interpolant)

I = eye(size(interpolant.D,1));
Dnew = inv(rangetrans.c*interpolant.D-rangetrans.a*I);

Cnew = -rangetrans.c*Dnew*interpolant.C;
Anew = interpolant.A + interpolant.B*Cnew;
Dnew = Dnew*(-rangetrans.d*interpolant.D+rangetrans.b*I);
Bnew = interpolant.B*(Dnew+rangetrans.d/rangetrans.c*I);

% Ryozo modified below. July 12, 2004.
interpolant.A = Anew;
interpolant.B = Bnew;
interpolant.C = Cnew;
interpolant.D = Dnew;
interpolant.sysf = tf(ss(Anew,Bnew,Cnew,Dnew,1));

%sys = balreal(ss(Anew,Bnew,Cnew,Dnew,1)); % Ryozo changed 2003/11/4
% sys = ss(Anew,Bnew,Cnew,Dnew,1);
% interpolant.A = sys.a;
% interpolant.B = sys.b;
% interpolant.C = sys.c;
% interpolant.D = sys.d;
% interpolant.sysf = tf(sys);